#!/usr/bin/env python
import sys
from datetime import datetime
from sqlalchemy import *
from sqlalchemy.orm import *
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.types import DateTime, Integer, String

import ClassAd

database = create_engine("sqlite:///database.db")
metadata = MetaData()
metadata.bind = database
Base = declarative_base()

def __parse_and(variable, relations):
    query = []
    for op, value in relations.items():
        if type(value) is list:
            for item in value:
                query.append("(%s %s '%s')" % (str(variable), str(op), str(item)))
        else:
            query.append("(%s %s '%s')" % (str(variable), str(op), str(value)))

    return " AND ".join(query)

def __parse_or(variable, relations):
    query = []
    for item in relations:
        if type(item) is dict:
            query.append("(%s)" % (__parse_and(variable, item)))
        else:
            query.append("(%s = '%s')" % (str(variable), str(item)))

    return " OR ".join(query)

def use_requirements(requirements):
    query = []
    for variable, value in requirements.items():
        if type(value) is dict:
            query.append("(%s)" % (__parse_and(variable, value)))
        elif type(value) is list:
            query.append("(%s)" % (__parse_or(variable, value)))
        else:
            query.append("(%s = '%s')" % (str(variable), str(value)))

    return " AND ".join(query)


def session():
    Session = sessionmaker(bind=database)
    return Session()

def install():
    Base.metadata.create_all(database)

class JobAd(Base):
    __tablename__ = 'job_queue'

    id = Column(Integer, primary_key=True)
    requirements = Column(String, nullable=False)
    status = Column(String)
    output = Column(String)
    error = Column(String)
    log = Column(String)
    executable = Column(String)
    arguments = Column(String)
    ram = Column(Integer)
    transfer = Column(String)
    submitter_ip = Column(String)
    local_id = Column(String)
    group_id = Column(Integer)
    queue = Column(Integer)

    def __init__(self, classad):
        self.requirements = ClassAd.encode(classad.get('requirements', ''))
        self.status = 'QUEUED'
        self.output = classad.get('output', '')
        self.error = classad.get('error', '')
        self.log = classad.get('log', '')
        self.executable = classad.get('executable', '')
        self.arguments = classad.get('arguments', '')
        self.ram = classad.get('ram', '')
        self.transfer = classad.get('transfer', '')
        self.submitter = classad.get('submitter', '')
        self.local_id = classad.get('local_id', '')
        self.group_id = 0
        self.queue = classad.get('queue', 1)



class MachineAd(Base):
    __tablename__ = 'machine_pool'
    ip = Column(String, primary_key=True)
    port = Column(String)
    arch = Column(String)
    os = Column(String)
    os_version = Column(String)
    cores = Column(Integer)
    ram = Column(Integer)
    subnet = Column(String)
    updated = Column(DateTime)

    def __init__(self, classad):
        self.ip = classad.get('ip', '')
        self.arch = classad.get('arch', '')
        self.os = classad.get('os', '')
        self.os_version = classad.get('os_version', '')
        self.cores = classad.get('cores', '')
        self.ram = classad.get('ram', '')
        self.subnet = classad.get('subnet', '')
        self.port = classad.get('port', '')
        self.updated = datetime.now()

class Database(object):

    def __init__(self):
        self.session = session()

    def jobs(self, requirements=None):
        query = self.session.query(JobAd)

        if requirements is None:
            return query
        else:
            return query.from_statement("SELECT * FROM job_queue WHERE %s" % use_requirements(requirements))

    def machines(self, requirements= None, limit=None):
        query = self.session.query(MachineAd)
        if requirements is None:
            return query
        else:
            statement = "SELECT * FROM machine_pool WHERE %s"
            if limit is not None:
                statement += " LIMIT %d" % (limit)

            return query.from_statement(statement % use_requirements(requirements))


    def get_job_info(self, jobid):
        results = self.session.query(JobAd).filter(JobAd.id == jobid)
        if results.count() > 0:
            return results[0]
        else:
            return None

    def get_machine_info(self, machineip):
        results = self.session.query(MachineAd).filter(MachineAd.ip == machineip)
        if results.count() > 0:
            return results[0]
        else:
            return None

    def insert_machine(self, ad):
        machine = MachineAd(ad)
        if self.get_machine_info(machine.ip) is None:
            self.session.add(machine)
            self.session.commit()

    def insert_job(self, job_ad):
        job = JobAd(job_ad)
        queue = job.queue

        self.session.add(job)
        self.session.commit()
        group_id = job.id
        job.group_id = group_id
        self.session.commit()

        for i in range(1, queue):
            queued_job = JobAd(job_ad)
            queued_job.group_id = group_id
            self.session.add(queued_job)
        self.session.commit()


    def update_job_status(self, jobid, status):
        if jobid is None or status is None:
            return

        job = self.get_job_info(jobid)
        if job is not None:
            job.status = status
            self.session.commit()

    def update_machine_status(self, machineip):
        if machineip is None:
            return

        machine = self.get_machine_info(machineip)
        if machine is not None:
            machine.updated = datetime.now()
            self.session.commit()

install()
if __name__ == "__main__":
   requirements = {'HAS_362' : "TRUE", 'version':[{'>':"2",'<':"4"},"6"], 'OS': ["LINUX", "WINNT51"] , 'arch' : {'!=':"INTEL"},  'cores': [{"<": "4"},{">":"6"}]}
   #print " Requirements : ", requirements
   print requirements
   print use_requirements(requirements)
