import threading, time, sys

import Config
import Database
import ClassAd
import DatabaseViewer

def fifo_scheduler(database):
    requirements = {'status':'QUEUED'}

    return database.jobs(requirements)

def find_matches(scheduler, database):
    print "Finding Matches..."
    print "=" * 72
    format = "%-10s%-16s%s"
    print format % ("Job ID", "Machine IP", "Requirements")
    print "-" * 72

    jobs = scheduler(database)
    for job in jobs:
        req = ClassAd.decode(job.requirements)
        machines = database.machines(req)

        matched = False
        machine = None
        for m in machines:
            if not matched:
                matched = True
                machine = m

        if matched:
            job.status = 'MATCHED'
            database.session.commit()

            print format % (job.id, machine.ip, job.requirements)
        else:
            print format % (job.id, "None Found", job.requirements)



class Negotiator(threading.Thread):
    def __init__(self, scheduler):
        threading.Thread.__init__(self)

        self.scheduler = scheduler
        self.database = Database.Database()

    def run(self):
        while True:
            find_matches(self.scheduler, self.database)
            time.sleep(Config.MATCHINTERVAL)

if __name__ == "__main__":
    if len(sys.argv) > 1:
        option = sys.argv[1]
    else:
        option = "once"

    if option == "repeat":
        neg = Negotiator(fifo_scheduler)
        neg.start()
    else:
        find_matches(fifo_scheduler, Database.Database())
    
            
