#!/usr/bin/env python

import select, sys, threading
from M2Crypto import SSL
import Config

class Client(object):
    def __init__(self, host, port, cert):
        self.host = host
        self.port = port
        self.size = 1024
        self.ctx = SSL.Context('sslv3')
        self.server = None
        
        self.ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, depth=9)
        if self.ctx.load_verify_locations(cert) != 1: 
            raise Exception('No CA certificates')

    def connect(self):
        self.server = SSL.Connection(self.ctx)
        self.server.connect((self.host, self.port))

    def disconnect(self):
        self.server.close()

    def run(self):
        pass

class ClientHandler(threading.Thread):
    def __init__(self,(client,address)):
        threading.Thread.__init__(self)
        self.client = client
        self.address = address
        self.size = 1024
        self.running = True

    def handle(self, data):
        sys.stdout.write("Client: '" + str(self.client) + "' said: '" + str(data) + "'\n")
        self.client.send(data)

    def run(self):
        while self.running:
            data = self.client.recv(self.size)
            if data:
                self.handle(data)
            else:
                self.client.close()
                self.running = False

class Server(object):
    def __init__(self, port, cert, key):
        self.host = ''
        self.port = port
        self.backlog = 5
        self.size = 1024
        self.server = None
        self.threads = []
        self.running = False
        self.cert = cert
        self.key = key

    def open(self):
        ctx = SSL.Context('sslv3')
        ctx.load_cert(certfile=self.cert, keyfile=self.key)
        self.server = SSL.Connection(ctx)
        self.server.bind(('', self.port))
        self.server.listen(5)
        self.running = True

    def run(self, handler=ClientHandler):
        self.open()
        sources = [self.server, sys.stdin]
        while self.running:
            inputready, outputready, exceptready = select.select(sources,[],[])

            for s in inputready:
                if s == self.server:
                    # handle the server socket
                    c = handler(self.server.accept())
                    c.start()
                    self.threads.append(c)

                if s == sys.stdin:
                    s.readline()
                    self.running = False

        # close all threads
        self.server.close()
        for c in self.threads:
            c.join()

        return False



if __name__ == "__main__":
    s = Server(Config.PORT, Config.CLIENTKEY, Config.SERVERKEY)
    s.run()



